import syft as sy
import utils
import pandas

class Student():

	"""
    Parameters:
      student_domain - Student's domain 
      student_user - user to connect to student_domain as data owner
      dataset_name - name for the dataset to be downloaded 
      tct_domain - TrustedCurator domain to download the data 
      tct_user - user to connect to tct_domain as data scientist 
      dataframe - pandas Dataframe which contains the data to upload 
      retries - amount of retries to wait for result 
      sleep_time - sleeping time for waiting result
      min_max_vals - a dict that indicates minimum value and maximum values for differential privacy range for the dataframe for each column 
    """
	def __init__(self, student_domain, student_user, dataset_name, tc_domain,
	 tc_user, dataframe, retries, sleep_time, min_max_vals={}):
		self.student_domain = student_domain
		self.student_user = student_user
		self.dataset_name = dataset_name
		self.tc_domain = tc_domain
		self.tc_user = tc_user
		self.dataframe = dataframe
		self.retries = retries
		self.sleep_time = sleep_time
		self.min_max_vals = min_max_vals


	def run(self):
		
		result = None
		try:
				print('login as owner') 
				domain = utils.login(self.student_domain, self.student_user)

				print('upload data') 
				utils.upload_dataset(domain, 
					self.dataset_name + '_student_query', 
					self.dataframe, self.min_max_vals)


				print('login as data scientist') 
				domain = utils.login(self.tc_domain, self.tc_user)

				print('download results') 
				result = utils.download_dataset(domain, 
					f'{self.dataset_name}_tc_result', 
					sleep_time=self.sleep_time, 
					retries=self.retries) 
	
				return result

		except Exception as ex:
			print(ex)

		
