import syft as sy
import utils
import time 
from joblib import dump, load
from sklearn.preprocessing import StandardScaler
import pandas as pd

class Teacher():
	"""
    Parameters:
      teacher_domain - Teacher's domain 
      teacher_user - user to connect to teacher_domain as data owner
      student_domain - Student's domain to download the data 
      student_user - user to connect to student_domain as data scientist 
      model_path - path where the saved model is 
      teacher_id - teacher identifier
      dataset_name - name for the dataset to be downloaded 
      classes - amount of classes
      min_max_vals - a dict that indicates minimum value and maximum values for differential privacy range for the result for each column 
    """
	def __init__(self, 
		teacher_domain,
		teacher_user, 
		student_domain, 
		student_user, 
		model_path, 
		teacher_id, 
		dataset_name,
		method = utils.Method.AVERAGE,
		classes=2,
		min_max_vals={}):
		self.teacher_domain = teacher_domain
		self.teacher_user = teacher_user
		self.student_domain = student_domain 
		self.student_user = student_user
		self.model = load(model_path) 
		self.teacher_id = teacher_id
		self.dataset_name = dataset_name
		self.method = method
		self.classes = classes
		self.min_max_vals = min_max_vals

	def update_model(self, model_path):
		self.model = load(model_path) 

	def get_domain(self):
		return self.teacher_domain

	def run(self):			
		try: 
			print('Logging to domains')
			teacher = utils.login(self.teacher_domain, self.teacher_user)
			student = utils.login(self.student_domain, self.student_user)
		
			print('Wait for dataset from student')
			df = utils.download_dataset(student, self.dataset_name + '_student_query')

			scaler = StandardScaler()
			values = scaler.fit_transform(df.values) 

			print('Make predictions')
			predictions = self.model.predict(values)

			if self.method == utils.Method.VOTING:
				result = pd.get_dummies(predictions, prefix='result')
				cols = result.columns
				if (self.classes>len(result.columns)): # fill with zeros a column for classes that are not in result
					for index in range(0, self.classes):
						if not (f'result_{index}' in a.columns):
							result[f'result_{index}'] = np.zeros(result.shape[0])

				print('Upload predictions')
				utils.upload_dataset(teacher, 
					f'{self.dataset_name}_teacher_{self.teacher_id}_result', 
					result,
					min_max_vals= self.min_max_vals)
				
			else:
				result = pd.DataFrame(predictions)
				result.columns = ['result']
				print(result)

				print('Upload predictions')
				
				utils.upload_dataset(teacher, 
					f'{self.dataset_name}_teacher_{self.teacher_id}_result', 
					result,
					min_max_vals=self.min_max_vals)
				
		except Exception as ex:
			print('error:', ex)

			