import syft as sy
import pandas as pd
import numpy as np

from sklearn.preprocessing import StandardScaler
from sklearn import datasets, linear_model
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
from joblib import dump, load
from sklearn.model_selection import train_test_split
from sklearn.tree import DecisionTreeClassifier


#login with default user 
domain = sy.login(
    email="info@openmined.org",
    password="changethis",
    url="http://localhost:8081"
)


#create Data Scientists users 
domain.users.create(name='DataScientist', 
	email='DataScientist@gmail.com', 
	password = 'DataScientist',
	budget= 10000000,
	role = "Data Scientist")

domain.users.create(name='DSStudent', 
	email='DSStudent@gmail.com', 
	password = 'DSStudent',
	budget= 10000000,
	role = "Data Scientist")

domain.users.create(name='DSTCT', 
	email='DSTC@gmail.com', 
	password = 'DSTCT',
	budget= 10000000,
	role = "Data Scientist")

domain.users.create(name='DST0', 
	email='DST0@gmail.com', 
	password = 'DST0',
	budget= 10000000,
	role = "Data Scientist")

domain.users.create(name='DST1', 
	email='DST1@gmail.com', 
	password = 'DST1',
	budget= 10000000,
	role = "Data Scientist")


### Train models ###

## Binary Classifier ##
df = pd.read_csv('../data/df_entrenar_modelo.csv', index_col=0)
print(df.columns)
x = df.iloc[:,0:8] # Features
y = df["Outcome"] # Lables


scaler = StandardScaler()
X_scaled = scaler.fit_transform(x.values)

model_0 = LogisticRegression(random_state=0, n_jobs=-1).fit(X_scaled , y.values)     
dump(model_0, 'model_classifier_0_2.joblib') 


model_1 = RandomForestClassifier(n_estimators=10, criterion='entropy').fit(X_scaled, y.values)
dump(model_1, 'model_classifier_1_2.joblib') 


## Multiclass Classifier##
  
iris = datasets.load_iris()
  
X = iris.data
y = iris.target
  
X_train, X_test, y_train, y_test = train_test_split(X, y, random_state = 0)


scaler = StandardScaler()
X_scaled = scaler.fit_transform(X_train)
  
dtree_model = DecisionTreeClassifier(max_depth = 2).fit(X_scaled, y_train)
dump(dtree_model, 'model_classifier_0_3.joblib') 

dtree_model = RandomForestClassifier(n_estimators=10, criterion='entropy').fit(X_scaled, y_train)
dump(dtree_model, 'model_classifier_1_3.joblib') 


## Regression ##
diabetes_x, diabetes_y = datasets.load_diabetes(return_X_y=True)
diabetes_x = diabetes_x[:, np.newaxis, 2]


diabetes_x_train = diabetes_x[:-20]
diabetes_x_test = diabetes_x[-20:]


diabetes_y_train = diabetes_y[:-20]
diabetes_y_test = diabetes_y[-20:]

scaler = StandardScaler()
diabetes_x_train = scaler.fit_transform(diabetes_x_train) 

regression = linear_model.LinearRegression()

regression.fit(diabetes_x_train, diabetes_y_train)
dump(regression, 'model_regression_0.joblib') 


regr_obj = RandomForestRegressor(max_depth=3, random_state=0)
regr_obj.fit(diabetes_x_train, diabetes_y_train)
dump(regr_obj, 'model_regression_1.joblib') 

