import syft as sy 
import time 
import numpy as np
import pandas as pd 
from enum import Enum

class User():
	def __init__(self, email, password, role):
		self.email = email
		self.password = password
		self.role = role


class Method(Enum):
	AVERAGE = 1
	VOTING = 2
	

def login(domain, user, retries=10, sleep_time=5):
	i = 0
	while(i<retries):
		try:
			return sy.login(email=user.email,password=user.password, url=domain)
		except:
			i+=1
			time.sleep(sleep_time)
			print(f'Retrying ..{i}')
	return None
 


def upload_dataset(domain, name, df, min_max_vals = {}):
	data_subjects = df.index.tolist() # one data subject per row 
	assets = {} # one asset per column 
	if min_max_vals=={}:
		for c in df.columns:
			min_max_vals[c] = (float(df[c].min()), float(df[c].max()))

	for c in df.columns:
	    assets[c] = sy.Tensor(df[c]).private(
							min_val=min_max_vals[c][0], 
							max_val=min_max_vals[c][1], 
							data_subjects=data_subjects
							)
	domain.load_dataset(
	    assets=assets,
	    name=name,
	    description=name
	) 



def download_dataset(domain, dataset_name, sleep_time=10, retries=10, sigma=2):
	i = 0
	while(i<retries):
		print(f'Privacy budget for domain {domain.privacy_budget}')
		print(f'Searching for dataset {dataset_name}')
		datasets = list(filter(lambda x: x.name==dataset_name, domain.datasets))
		if len(datasets)>0:
			data = datasets[0]
			cols = [item['name'] for item in data.assets]
			df = pd.DataFrame()
			pointers = []
			for c in cols:
				published_result = data[c].publish(sigma=sigma)
				pointers.append(published_result)

			i = 0
			for c in cols:
				df[c] = pointers[i].get()
				i+=1

			return df
		else:
			i+=1
			time.sleep(sleep_time)
			print(f'Waiting for dataset {dataset_name}..')


def get_pointer(domain, dataset_name, colname, sleep_time=10, retries=10, sigma=2):
	i = 0
	while(i<retries):
		print(f'Searching for dataset {dataset_name}')
		datasets = list(filter(lambda x: x.name==dataset_name, domain.datasets))
		if len(datasets)>0:
			data = datasets[0]
			return data[colname]
		else:
			i+=1
			time.sleep(sleep_time)
			print(f'Waiting for dataset {dataset_name}..')



def delete_dataset(domain, dataset_name):
	datasets = list(filter(lambda x: x['name']==dataset_name, domain.datasets.all()))
	if len(datasets)>0:
		for i in range(len(datasets)):
			domain.datasets.delete(dataset_id=datasets[i]['id'])









